@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblber/debug.c,v 1.9.2.5 2004/02/23 22:08:05 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblber/debug.c,v 1.9.2.7 2005/01/20 17:01:00 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2005 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -146,8 +146,6 @@ void lutil_log_int(
 	time_t now;
 	struct tm *today;
 #endif
-	size_t i;
-	char * tmp;
 
 #ifdef LDAP_SYSLOG
 	/* we're configured to use syslog */
